part of aargul_crud;

class AargulCrudField<T> extends EventEmitter implements ValueNotifier {
  final String type;
  final bool editable;
  final bool listable;
  final T? initialValue;
  final Widget Function(AargulCrudField<T> field)? customBuilder;
  final void Function(AargulCrudField<T> field)? onChanged;
  final Widget Function(
          AargulCrudField<T> field, dynamic value, Map<String, dynamic> values)?
      renderCell;
  final Widget Function(AargulCrudField<T> field)? renderColunm;
  final AargulCrudTableColumnSize? columnSize;
  final AargulCrudColumnSize? xs;
  final AargulCrudColumnSize? sm;
  final AargulCrudColumnSize? md;
  final AargulCrudColumnSize? lg;
  final AargulCrudColumnSize? xl;
  final TextStyle? labelStyle;

  final Map<String, AargulCrudField> _fields = {};

  dynamic _options = {};
  dynamic get options => _options;
  set options(dynamic options) {
    _options = options;
    notifyListeners();
  }

  bool _hasInput = true;
  bool get hasInput => _hasInput;
  set hasInput(bool hasInput) {
    _hasInput = hasInput;
    notifyListeners();
  }

  bool _disabled = false;
  bool get disabled => _disabled;
  set disabled(bool disabled) {
    _disabled = disabled;
    notifyListeners();
  }

  String? _label;
  String? get label => _label;
  set label(String? label) {
    _label = label;
    notifyListeners();
  }

  String? _placeholder;
  String? get placeholder => _placeholder;
  set placeholder(String? placeholder) {
    _placeholder = placeholder;
    notifyListeners();
  }

  T? _value;
  @override
  T? get value => _value;

  @override
  set value(newValue) {
    if (_value == newValue) {
      return;
    }
    _value = newValue;
    notifyListeners();
  }

  AargulCrudField? formField(String fieldName) {
    return _fields[fieldName];
  }

  AargulCrudField({
    String? placeholder,
    String? label,
    bool disabled = false,
    bool hasInput = true,
    AargulCrudFieldType type = AargulCrudFieldType.input,
    dynamic options = const {},
    this.editable = true,
    this.listable = true,
    this.customBuilder,
    this.labelStyle,
    this.renderCell,
    this.renderColunm,
    this.columnSize,
    this.onChanged,
    this.xs,
    this.sm,
    this.md,
    this.lg,
    this.xl,
    this.initialValue,
  })  : type = type.name,
        _label = label,
        _placeholder = placeholder,
        _options = options,
        _hasInput = hasInput,
        _disabled = disabled,
        _value = initialValue;

  /// IMPLEMENT VALUE LISTANABLE
  final Map<VoidCallback, EventListener> _listeners = {};
  Timer _notifyListeners = Timer(Duration.zero, () {});
  @override
  void addListener(VoidCallback listener) {
    _listeners[listener]?.cancel();
    _listeners[listener] = on(_fieldUpdatedEvent, (s) => listener());
  }

  @override
  void dispose() {
    for (var value in listeners) {
      value.cancel();
    }
    _listeners.clear();
    listeners.clear();
  }

  @override
  bool get hasListeners => _listeners.isNotEmpty;

  @override
  void notifyListeners() {
    _notifyListeners.cancel();
    _notifyListeners = Timer(const Duration(milliseconds: 1), () {
      emit(_fieldUpdatedEvent);
    });
    onChanged?.call(this); // notify the changement
  }

  @override
  void removeListener(VoidCallback listener) {
    _listeners[listener]?.cancel();
    _listeners.remove(listener);
  }
}
