part of aargul_crud;

extension ExtensionAargulCrudManagerGrid on AargulCrudManager {
  Widget grid({
    required AargulCrudGridCell Function(Map<String, dynamic> values)
        itemBuilder,
    WrapAlignment alignment = WrapAlignment.start,
    WrapCrossAlignment crossAlignment = WrapCrossAlignment.start,
    WrapAlignment runAlignment = WrapAlignment.start,
    Axis direction = Axis.horizontal,
    VerticalDirection verticalDirection = VerticalDirection.down,
    double spacing = 0,
    double runSpacing = 0,
    Clip clipBehavior = Clip.none,
    EdgeInsetsGeometry? padding,
    EdgeInsetsGeometry? margin,
    double? width,
    double? height,
    BoxDecoration? decoration,
    Decoration? foregroundDecoration,
    TextDirection? textDirection,
    BoxConstraints? constraints,
  }) {
    ValueNotifier<(int, int, int?, List<Map<String, dynamic>>?)> state =
        ValueNotifier((0, 10, null, null));
    refresh((int, int, int?, List<Map<String, dynamic>>?) value) async {
      if (value.$3 != null && (value.$4 ?? []).length >= (value.$3!)) return;
      var res = await options.read?.call(value.$1, value.$2);
      if (res != null) {
        state.value = (
          value.$1,
          value.$2,
          res.totalRows,
          [...(value.$4 ?? []), ...res.rows]
        );
      } else {
        state.value =
            (value.$1, value.$2, value.$3 ?? value.$4?.length ?? 0, value.$4);
      }
    }

    refresh(state.value);
    return ValueListenableBuilder(
      valueListenable: state,
      builder: (context, value, child) {
        return Container(
          width: width ?? MediaQuery.of(context).size.width,
          height: height,
          decoration: decoration,
          constraints: constraints,
          foregroundDecoration: foregroundDecoration,
          padding: padding,
          margin: margin,
          child: material.RefreshIndicator(
            onRefresh: () async {
              await refresh(value);
            },
            child: SingleChildScrollView(
              physics: const AlwaysScrollableScrollPhysics(),
              child: SpGrid(
                spacing: spacing,
                runSpacing: runSpacing,
                direction: direction,
                runAlignment: runAlignment,
                crossAlignment: crossAlignment,
                alignment: alignment,
                width: width ?? MediaQuery.of(context).size.width,
                children: [
                  ...(value.$4 ?? []).map((item) {
                    return itemBuilder(item);
                  }).toList(),
                  if (value.$3 != null &&
                      value.$4 != null &&
                      value.$4!.length < value.$3!)
                    AargulCrudGridCell(
                      xs: AargulCrudColumnSize.col12,
                      child: VisibilityDetector(
                          key: const Key("load_more_indicator"),
                          onVisibilityChanged: (VisibilityInfo info) async {
                            // show loader inficator
                            state.value =
                                (value.$1, value.$2, null, value.$4 ?? []);
                            // load next page
                            await refresh((
                              value.$1 + value.$2,
                              value.$2,
                              value.$3,
                              value.$4 ?? []
                            ));
                          },
                          child: Button(
                            onPressed: () async {
                              // show loader inficator
                              state.value =
                                  (value.$1, value.$2, null, value.$4 ?? []);
                              // load next page
                              await refresh((
                                value.$1 + value.$2,
                                value.$2,
                                value.$3,
                                value.$4 ?? []
                              ));
                            },
                            child: const Text("Charger plus"),
                          )),
                    ),
                  if (value.$3 == null)
                    AargulCrudGridCell(
                      sm: AargulCrudColumnSize.col12,
                      child: const SizedBox(
                        height: 48,
                        child: Center(
                          child: SizedBox.square(
                            dimension: 48,
                            child: ProgressRing(),
                          ),
                        ),
                      ),
                    ),
                ],
              ),
            ),
          ),
        );
      },
    );
  }
}

class AargulCrudGridCellOrder extends SpOrder {
  const AargulCrudGridCellOrder({
    super.xs,
    super.sm,
    super.md,
    super.lg,
    super.xl,
  });
}

class AargulCrudGridCell extends SpGridItem {
  AargulCrudGridCell(
      {AargulCrudGridCellOrder? order,
      BoxDecoration? decoration,
      AargulCrudColumnSize? xs,
      AargulCrudColumnSize? sm,
      AargulCrudColumnSize? md,
      AargulCrudColumnSize? lg,
      AargulCrudColumnSize? xl,
      required Widget child})
      : super(
            child: child,
            sm: (sm != null ? (sm.index + 1) : null),
            md: (md != null ? (md.index + 1) : null),
            lg: (lg != null ? (lg.index + 1) : null),
            xl: (xl != null ? (xl.index + 1) : null),
            xs: (xs != null ? (xs.index + 1) : null) ?? 12,
            order: SpOrder(
              sm: order?.sm,
              md: order?.md,
              lg: order?.lg,
              xl: order?.xl,
              xs: order?.xs,
            ),
            decoration: decoration);
}
