part of aargul_crud;

enum AargulCrudTableColumnSize { S, M, L }

extension ExtensionAargulCrudManagerTable on AargulCrudManager {
  Widget table([List<AargulCrudManagerAction> actions = const []]) {
    return AsyncPaginatedDataTable2(
        source: _DataSourceAsync(options, actions),
        columns: [
          ...options.fields
              .map((fieldName, field) {
                return MapEntry(
                    fieldName,
                    field.listable
                        ? DataColumn2(
                            size: ColumnSize.values[
                                field.columnSize?.index ?? ColumnSize.M.index],
                            label: field.renderColunm?.call(field) ??
                                Text(field.label ?? (fieldName.toUpperCase())),
                          )
                        : null);
              })
              .values
              .whereType<DataColumn2>()
              .toList(),
          if (actions.isNotEmpty)
            const DataColumn2(
              size: ColumnSize.S,
              label: SizedBox.shrink(),
            ),
        ]);
  }
}

class _DataSourceAsync extends AsyncDataTableSource {
  final AargulCrudOptions options;
  final List<AargulCrudManagerAction> actions;
  _DataSourceAsync(this.options, this.actions);

  @override
  Future<AsyncRowsResponse> getRows(int startIndex, int count) async {
    assert(startIndex >= 0);

    AargulCrudDataResponse totalRecords =
        await (options.read?.call(startIndex, count) ??
            Future.value(AargulCrudDataResponse(0, [])));

    var r = AsyncRowsResponse(
        totalRecords.totalRows,
        totalRecords.rows.map((row) {
          var key = options.getKey != null
              ? ValueKey<String>(options.getKey!.call(row))
              : null;
          return material.DataRow(
            key: key,
            // selected: key != null
            //     ? selectionRowKeys.where((k) => k == key).isNotEmpty
            //     : false,
            // onSelectChanged: (value) {
            //   if (value != null && key != null) {
            //     setRowSelection(key, value);
            //   }
            // },
            cells: [
              ...options.fields
                  .map((fieldName, field) {
                    return MapEntry(
                      fieldName,
                      !field.listable
                          ? null
                          : material.DataCell(
                              field.renderCell
                                      ?.call(field, row[fieldName], row) ??
                                  AargulCrudManager._cellBuilder[field.type]
                                      ?.call(fieldName, field, row) ??
                                  Text("${row[fieldName]}"),
                            ),
                    );
                  })
                  .values
                  .whereType<material.DataCell>()
                  .toList(),
              if (actions.isNotEmpty)
                material.DataCell(Builder(
                  builder: (context) {
                    final menuController = FlyoutController();

                    return FlyoutTarget(
                        controller: menuController,
                        child: Button(
                          child: const Icon(FluentIcons.more_vertical),
                          onPressed: () {
                            menuController.showFlyout(
                              barrierDismissible: true,
                              dismissOnPointerMoveAway: false,
                              dismissWithEsc: true,
                              builder: (context) {
                                return MenuFlyout(
                                    items: actions
                                        .map(
                                          (action) => MenuFlyoutItem(
                                            leading: Icon(action.icon),
                                            text: Text(action.label),
                                            onPressed: action.onPressed != null
                                                ? () {
                                                    Flyout.of(context).close();
                                                    Future.microtask(() =>
                                                        action.onPressed
                                                            ?.call(row));
                                                  }
                                                : null,
                                          ),
                                        )
                                        .toList());
                              },
                            );
                          },
                        ));
                  },
                )),
            ],
          );
        }).toList());

    return r;
  }
}
